import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, Tabs, Popconfirm, notification, Modal, Typography, Spin, message, Button, Input } from 'antd'
import { StopOutlined, CheckCircleOutlined, TableOutlined, EditOutlined, SwapOutlined, DeleteOutlined, CopyOutlined, BorderOutlined, SnippetsOutlined } from '@ant-design/icons'
import moment from 'moment'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import { formatSearch, joinMainSearchkey } from '@/utils/utils-custom.js'
import asyncComponent from '@/utils/asyncComponent'
import ColForm from './columnform'
import CustomScriptsForm from './customscript'
import SettingForm from './settingform'
import PasteForm from '@/templates/zshare/pasteform'
import SettingUtils from './utils'
import MKEmitter from '@/utils/events.js'
import MinView from '@/assets/img/minview.png'
import './index.scss'

const { TabPane } = Tabs
const { Paragraph } = Typography
const { Search } = Input
const { confirm } = Modal

const ExcelOut = asyncComponent(() => import('./excelout'))
const CodeMirror = asyncComponent(() => import('@/templates/zshare/codemirror'))
const FieldsComponent = asyncComponent(() => import('@/templates/sharecomponent/fieldscomponent'))
const EditTable = asyncComponent(() => import('@/templates/zshare/editTable'))

class VerifyCard extends Component {
  static propTpyes = {
    config: PropTypes.object,
    updRecord: PropTypes.func
  }

  state = {
    columns: [],
    debugId: '',
    subColumns: [],
    activeKey: 'setting',
    loading: false,
    colLoading: false,
    searchKey: '',
    usefulfields: '',
    defaultsql: '',
    systemScripts: [],
    median: {},
    visible: false,
    editLineId: '',
    pvisible: false,
    reload: false,
    script: null,
    oriConfig: null,
    scriptValue: '',
    cols: null,
    colColumns: [
      {
        title: '名称',
        dataIndex: 'label',
        inputType: 'input',
        editable: true,
        searchable: true,
        width: '28%'
      },
      {
        title: '字段',
        dataIndex: 'field',
        inputType: 'input',
        editable: true,
        unique: true,
        strict: true,
        searchable: true,
        copy: true,
        rules: [{
          pattern: /^[\u4E00-\u9FA50-9a-zA-Z_]*$/ig,
          message: '字段名只允许包含数字、字母、汉字以及_'
        }],
        width: '28%'
      },
      {
        title: '数据类型',
        dataIndex: 'datatype',
        inputType: 'select',
        options: [
          { value: 'Nvarchar(10)', text: 'Nvarchar(10)' },
          { value: 'Nvarchar(20)', text: 'Nvarchar(20)' },
          { value: 'Nvarchar(50)', text: 'Nvarchar(50)' },
          { value: 'Nvarchar(100)', text: 'Nvarchar(100)' },
          { value: 'Nvarchar(256)', text: 'Nvarchar(256)' },
          { value: 'Nvarchar(512)', text: 'Nvarchar(512)' },
          { value: 'Nvarchar(1024)', text: 'Nvarchar(1024)' },
          { value: 'Nvarchar(2048)', text: 'Nvarchar(2048)' },
          { value: 'Nvarchar(4000)', text: 'Nvarchar(4000)' },
          { value: 'Nvarchar(max)', text: 'Nvarchar(max)' },
          { value: 'Int', text: 'Int' },
          { value: 'Decimal(18,0)', text: 'Decimal(18,0)' },
          { value: 'Decimal(18,1)', text: 'Decimal(18,1)' },
          { value: 'Decimal(18,2)', text: 'Decimal(18,2)' },
          { value: 'Decimal(18,3)', text: 'Decimal(18,3)' },
          { value: 'Decimal(18,4)', text: 'Decimal(18,4)' },
          { value: 'Decimal(18,5)', text: 'Decimal(18,5)' },
          { value: 'Decimal(18,6)', text: 'Decimal(18,6)' },
          { value: 'Decimal(18,7)', text: 'Decimal(18,7)' },
          { value: 'Decimal(18,8)', text: 'Decimal(18,8)' },
          { value: 'Decimal(18,9)', text: 'Decimal(18,9)' },
          { value: 'Decimal(18,10)', text: 'Decimal(18,10)' },
          { value: 'Decimal(18,11)', text: 'Decimal(18,11)' },
          { value: 'Decimal(18,12)', text: 'Decimal(18,12)' },
          { value: 'Decimal(18,13)', text: 'Decimal(18,13)' },
          { value: 'Decimal(18,14)', text: 'Decimal(18,14)' },
          { value: 'Decimal(18,15)', text: 'Decimal(18,15)' },
          { value: 'Decimal(18,16)', text: 'Decimal(18,16)' },
          { value: 'Decimal(18,17)', text: 'Decimal(18,17)' },
          { value: 'Decimal(18,18)', text: 'Decimal(18,18)' },
          { value: 'date', text: 'date' },
          { value: 'datetime', text: 'datetime' },
        ],
        editable: true,
        width: '25%',
      }
    ],
    scriptsColumns: [
      {
        title: 'SQL',
        dataIndex: 'sql',
        width: '65%',
        render: (text) => {
          let title = text.match(/^\s*\/\*.+\*\//)
          title = title && title[0] ? title[0] : ''
          let _text = title ? text.replace(title, '') : text

          return (
            <div>
              {title ? <span style={{color: '#a50'}}>{title}<span style={{fontSize: '12px', marginLeft: '5px'}}>{_text.length}</span></span> : null}
              <Paragraph copyable={{ text: text }} ellipsis={{ rows: 4, expandable: true }}>{_text}</Paragraph>
            </div>
          )
        }
      },
      {
        title: '执行位置',
        dataIndex: 'position',
        width: '10%',
        render: (text, record) => {
          if (record.position === 'back') {
            return <span style={{color: '#1890ff'}}>后置</span>
          } else {
            return <span style={{color: '#26C281'}}>前置</span>
          }
        }
      },
      {
        title: '状态',
        dataIndex: 'status',
        width: '10%',
        render: (text, record) => record.status === 'false' ?
          (
            <div style={{color: '#ff4d4f'}}>
              禁用
              <StopOutlined style={{marginLeft: '5px'}} />
            </div>
          ) :
          (
            <div style={{color: '#26C281'}}>
              启用
              <CheckCircleOutlined style={{marginLeft: '5px'}}/>
            </div>
          )
      },
      {
        title: '操作',
        align: 'center',
        width: '15%',
        dataIndex: 'operation',
        render: (text, record) =>
          (<div style={{textAlign: 'center'}}>
            <span className="operation-btn" title="编辑" onClick={() => this.handleEdit(record)} style={{color: '#1890ff'}}><EditOutlined /></span>
            <span className="operation-btn" title="状态切换" onClick={() => this.handleStatus(record)} style={{color: '#8E44AD'}}><SwapOutlined /></span>
            <Popconfirm
              overlayClassName="popover-confirm"
              title="确定删除吗?"
              onConfirm={() => this.deleteScript(record)
            }>
              <span className="operation-btn" style={{color: '#ff4d4f'}}><DeleteOutlined /></span>
            </Popconfirm>
          </div>)
      }
    ]
  }

  UNSAFE_componentWillMount() {
    const { config, mainSearch } = this.props

    let search = config.search || []

    if (config.setting.useMSearch === 'true') { // 使用主搜索条件
      search = [...search, ...mainSearch]
    }

    let _setting = fromJS(config.setting).toJS()
    let scripts = config.scripts ? fromJS(config.scripts).toJS() : []

    // if (window.GLOB.funcs && window.GLOB.funcs.length > 0) {
    //   window.GLOB.funcs.forEach(m => {
    //     let reg = new RegExp(`\\/\\*\\$ex@${m.func_code}-begin\\*\\/[\\s\\S]+\\/\\*@ex\\$-end\\*\\/`, 'ig')
    //     if (_setting.dataresource) {
    //       _setting.dataresource = _setting.dataresource.replace(reg, `$ex@${m.func_code}@ex$`)
    //     }
    //     scripts && scripts.forEach(item => {
    //       item.sql = item.sql.replace(reg, `$ex@${m.func_code}@ex$`)
    //     })
    //   })
    // }

    let columns = config.columns ? fromJS(config.columns).toJS() : []
    let subColumns = config.subColumns ? fromJS(config.subColumns).toJS() : []

    if (!config.fixedCol) {
      columns.reverse()
    }
    subColumns.reverse()
    
    columns.forEach(col => {
      if (!col.datatype) return
      if (/^nvarchar/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^nvarchar/, 'Nvarchar')
      } else if (/^decimal/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^decimal/, 'Decimal')
      } else if (/^int/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^int/, 'Int')
      }
    })
    subColumns.forEach(col => {
      if (!col.datatype) return
      if (/^nvarchar/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^nvarchar/, 'Nvarchar')
      } else if (/^decimal/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^decimal/, 'Decimal')
      } else if (/^int/.test(col.datatype)) {
        col.datatype = col.datatype.replace(/^int/, 'Int')
      }
    })

    this.setState({
      scripts,
      columns: columns,
      subColumns: subColumns,
      setting: _setting,
      median: _setting,
      searches: formatSearch(search),
      searchKey: '',
      debugId: _setting.debugId || '',
      oriConfig: fromJS({
        scripts,
        columns: columns,
        subColumns: subColumns,
        setting: _setting
      }).toJS()
    })

    this.getsysScript()
  }

  getsysScript = () => {
    if (sessionStorage.getItem('mk_sys_scripts')) {
      this.setState({
        systemScripts: JSON.parse(sessionStorage.getItem('mk_sys_scripts'))
      })
      return
    }

    let _scriptSql = `Select distinct func+Remark as funcname,longparam, s.Sort from  s_custom_script s inner join (select OpenID from sapp where ID=@Appkey@) p on s.openid = case when s.appkey='' then s.openid else p.OpenID end order by s.Sort`

    _scriptSql = Utils.formatOptions(_scriptSql, 'x')

    let _sParam = {
      func: 'sPC_Get_SelectedList',
      LText: _scriptSql,
      obj_name: 'data',
      arr_field: 'funcname,longparam',
      exec_type: 'x'
    }
    
    _sParam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
    _sParam.secretkey = Utils.encrypt('', _sParam.timestamp)
    _sParam.open_key = Utils.encryptOpenKey(_sParam.secretkey, _sParam.timestamp) // 云端数据验证
    
    Api.getCloudConfig(_sParam).then(res => {
      if (res.status) {
        let _scripts = res.data.map(item => {
          return {
            name: item.funcname,
            value: window.decodeURIComponent(window.atob(item.longparam))
          }
        })

        sessionStorage.setItem('mk_sys_scripts', JSON.stringify(_scripts))

        this.setState({
          systemScripts: _scripts
        })
      } else {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
      }
    })
  }

  columnChange = (values, resolve) => {
    const { columns } = this.state

    let fields = columns.map(item => item.field.toLowerCase())
    if (fields.includes(values.field.toLowerCase())) {
      notification.warning({
        top: 92,
        message: '字段已存在！',
        duration: 5
      })
      return
    }

    resolve()

    values.uuid = Utils.getuuid()

    this.setState({ columns: [values, ...columns] })
  }

  subColumnChange = (values, resolve) => {
    const { subColumns } = this.state

    let fields = subColumns.map(item => item.field.toLowerCase())
    if (fields.includes(values.field.toLowerCase())) {
      notification.warning({
        top: 92,
        message: '字段已存在！',
        duration: 5
      })
      return
    }

    resolve()

    values.uuid = Utils.getuuid()

    this.setState({ subColumns: [values, ...subColumns] })
  }

  deleteScript = (record) => {
    const { script, scripts } = this.state

    if (script && script.uuid === record.uuid) {
      this.setState({script: null})
    }
    
    this.setState({ scripts: scripts.filter(item => item.uuid !== record.uuid) })
  }

  handleEdit = (record) => {
    this.scriptsForm.edit(record)
    
    // let node = document.getElementById('mk-scripts-tabpane').parentNode
    let node = document.getElementById('mk-scripts-tabpane')

    if (node && node.scrollTop) {
      let inter = Math.ceil(node.scrollTop / 10)

      let timer = setInterval(() => {
        if (node.scrollTop - inter > 0) {
          node.scrollTop = node.scrollTop - inter
        } else {
          node.scrollTop = 0
          clearInterval(timer)
        }
      }, 10)
    }
  }

  handleStatus = (record) => {
    let scripts = fromJS(this.state.scripts).toJS()
    record.status = record.status === 'false' ? 'true' : 'false'

    scripts = scripts.map(item => {
      if (item.uuid === record.uuid) {
        return record
      } else {
        return item
      }
    })

    this.setState({ scripts })
  }

  scriptsChange = (values) => {
    let scripts = fromJS(this.state.scripts).toJS()

    if (values.uuid) {
      scripts = scripts.map(item => {
        if (item.uuid === values.uuid) {
          values.$index = item.$index || ''
          return values
        } else {
          return item
        }
      })
    } else {
      scripts.push(values)
    }

    return new Promise((resolve, reject) => {
      this.sqlverify(resolve, reject, 'script', scripts)
    })
  }

  changeScripts = (scripts) => {
    this.setState({ scripts })
  }

  scriptSubmit = (values) => {
    let scripts = fromJS(this.state.scripts).toJS()

    if (values.uuid) {
      scripts = scripts.map(item => {
        if (item.uuid === values.uuid) {
          return values
        } else {
          return item
        }
      })
    } else {
      values.uuid = Utils.getuuid()
      scripts.push(values)
      MKEmitter.emit('editLineId', values.uuid)
    }

    this.setState({ scripts })
  }

  changeTab = (val) => {
    const { config, mainSearch } = this.props
    const { activeKey } = this.state

    this.setState({loading: true})
    if (activeKey === 'setting') {
      this.settingForm.handleConfirm().then(res => {
        let search = config.search || []

        if (res.useMSearch === 'true') { // 使用主搜索条件
          search = [...search, ...mainSearch]
        }

        this.setState({
          searches: formatSearch(search),
          setting: res
        }, () => {
          this.sqlverify(() => { // 验证成功
            this.setState({
              activeKey: val,
              loading: false
            })
            this.getdefaultSql()
          }, () => {             // 验证失败
            this.setState({
              activeKey: val,
              loading: false
            })
            this.getdefaultSql()
          }, 'change')
        })
      }, () => {
        this.setState({loading: false})
      })
    } else if (activeKey === 'columns') {
      if (this.datasource && this.datasource.state.editingKey) {
        notification.warning({
          top: 92,
          message: '字段未保存，请保存后切换！',
          duration: 5
        })
        this.setState({
          loading: false
        })
        return
      }

      this.setState({
        activeKey: val,
        loading: false
      })
      this.getdefaultSql()
    } else if (activeKey === 'subcolumns') {
      if (this.subdatasource && this.subdatasource.state.editingKey) {
        notification.warning({
          top: 92,
          message: '字段未保存，请保存后切换！',
          duration: 5
        })
        this.setState({
          loading: false
        })
        return
      }

      this.setState({
        activeKey: val,
        loading: false
      })
    } else if (activeKey === 'scripts') {
      let _loading = false
      if (this.scriptsForm && this.scriptsForm.state.editItem) {
        _loading = true
      } else if (this.scriptsForm && this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))) {
        _loading = true
      }

      if (_loading) {
        notification.warning({
          top: 92,
          message: '存在未保存脚本！',
          duration: 5
        })
      }

      this.setState({
        activeKey: val,
        loading: false
      })
    }
  }

  getdefaultSql = () => {
    const { columns, setting, searches } = this.state
    let defaultsql = ''
    let arr_field = columns.map(col => col.field).join(',')

    let _search = joinMainSearchkey(searches)

    if (setting.dataresource) {
      let _dataresource = setting.dataresource

      if (/\s/.test(_dataresource)) {
        _dataresource = '(' + _dataresource + ') tb'
      }

      defaultsql = `select top @pageSize@ ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by @orderBy@) as rows from ${_dataresource} ${_search}) tmptable where rows > (@pageSize@ * (@pageIndex@ - 1)) order by tmptable.rows`
    }

    this.setState({defaultsql})
  }

  submitDataSource = () => {
    const { config, mainSearch } = this.props
    const { activeKey, setting, columns, subColumns, scripts, cols, median, searches } = this.state

    if (config.subtype === 'dualdatacard') {
      let arr = columns.map(col => col.field.toLowerCase())
      let _arr = []
      subColumns.forEach(col => {
        if (arr.includes(col.field.toLowerCase())) {
          _arr.push(col.field)
        }
      })

      if (_arr.length > 0) {
        notification.warning({
          top: 92,
          message: '子表中字段' + _arr.join('、') + '与主表字段重复！',
          duration: 5
        })
        return Promise.reject()
      }
    } else if (config.subtype === 'editable') {
      let _arr = []
      columns.forEach(col => {
        let key = col.field.toLowerCase()
        if (['jskey', 'data_type', 'bid'].includes(key)) {
          _arr.push(col.field)
        }
      })

      if (_arr.length > 0) {
        notification.warning({
          top: 92,
          message: '字段集中不可使用' + _arr.join('、') + '！注：此字段为提交时的系统字段。',
          duration: 5
        })
        return Promise.reject()
      }
    }

    if (median.interType === 'system') {
      let _loading = false
      if (this.scriptsForm && this.scriptsForm.state.editItem) {
        _loading = true
      } else if (this.scriptsForm && this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))) {
        _loading = true
      }
  
      if (_loading) {
        notification.warning({
          top: 92,
          message: '存在未保存脚本，请点击确定保存，或点击取消放弃修改！',
          duration: 5
        })
        this.setState({activeKey: 'scripts'})
        return Promise.reject()
      }
    }

    return new Promise((resolve, reject) => {
      if (activeKey === 'setting') {
        this.settingForm.handleConfirm().then(res => {
          let search = config.search || []

          if (res.useMSearch === 'true') { // 使用主搜索条件
            search = [...search, ...mainSearch]
          }
          let _searches = formatSearch(search)

          this.setState({
            searches: _searches,
            setting: res
          }, () => {
            this.sqlverify(() => { resolve({setting: res, columns, subColumns, scripts, cols, searches: _searches }) }, reject, 'submit')
          })
        }, () => {
          reject()
        })
      } else if (activeKey === 'columns') {
        if (this.datasource && this.datasource.state.editingKey) {
          notification.warning({
            top: 92,
            message: '字段未保存，请保存后提交！',
            duration: 5
          })
          reject()
          return
        }
        this.sqlverify(() => { resolve({setting, columns, subColumns, scripts, cols, searches }) }, reject, 'submit')
      } else if (activeKey === 'subcolumns') {
        if (this.subdatasource && this.subdatasource.state.editingKey) {
          notification.warning({
            top: 92,
            message: '字段未保存，请保存后提交！',
            duration: 5
          })
          reject()
          return
        }
        this.sqlverify(() => { resolve({setting, columns, subColumns, scripts, cols, searches }) }, reject, 'submit')
      } else if (activeKey === 'scripts') {
        this.sqlverify(() => { resolve({setting, columns, subColumns, scripts, cols, searches }) }, reject, 'submit')
      }
    })
  }

  closeDataSource = (callback) => {
    const { setting, columns, subColumns, scripts, oriConfig } = this.state

    let line = scripts.map(item => item.sql).join('')
    line += columns.map(item => item.field + item.datatype).join('')
    line += subColumns.map(item => item.field + item.datatype).join('')
    line += setting.interType + (setting.dataresource || '') + (setting.order || '') + (setting.primaryKey || '')

    let _line = oriConfig.scripts.map(item => item.sql).join('')
    _line += oriConfig.columns.map(item => item.field + item.datatype).join('')
    _line += oriConfig.subColumns.map(item => item.field + item.datatype).join('')
    _line += oriConfig.setting.interType + (oriConfig.setting.dataresource || '') + (oriConfig.setting.order || '') + (oriConfig.setting.primaryKey || '')

    if (line !== _line) {
      confirm({
        content: '数据源已修改，确定取消吗？',
        onOk() {
          callback()
        },
        onCancel() {}
      })
    } else {
      callback()
    }
  }

  sqlverify = (resolve, reject, type, testScripts) => {
    const { config } = this.props
    const { columns, setting, scripts, searches, debugId, subColumns } = this.state

    let _scripts = scripts.filter(item => item.status !== 'false')

    if (testScripts) {
      _scripts = testScripts.filter(item => item.status !== 'false')
    }
    if (type !== 'change' && setting.interType === 'system' && setting.execute === 'false' && _scripts.length === 0) {
      notification.warning({
        top: 92,
        message: '不执行默认sql时，请添加自定义脚本！',
        duration: 5
      })
      reject()
      return
    }

    if (type === 'change' && columns.length === 0) {
      reject()
      return
    }

    if ((setting.interType === 'system' && setting.execute !== 'false') || _scripts.length > 0) {
      let _columns = columns
      if (config.subtype === 'dualdatacard') {
        _columns = [...columns, ...subColumns]
      }

      let r = SettingUtils.getDebugSql(setting, _scripts, _columns, searches, config.subtype, config.hasExtend)

      if (r.custompage && setting.laypage === 'true' && _columns.findIndex(col => col.field === 'mk_total') === -1) {
        if (config.subtype !== 'basetable') {
          Modal.warning({
            title: `数据源或自定义脚本中使用自定义分页排序时，请在字段集中添加 mk_total。`,
            okText: '知道了',
            onOk: () => {
              reject()
            }
          })
          return
        } else {
          notification.warning({
            top: 92,
            message: '数据源或自定义脚本中使用自定义分页排序时，请在显示列中添加 mk_total。',
            duration: 5
          })
        }
      }

      if (debugId === r.debugId) {
        resolve()
        return
      }

      if (r.errors) {
        notification.warning({
          top: 92,
          message: r.errors,
          duration: 5
        })
      }

      Api.sDebug(r.sql).then(result => {
        if (result.status || result.ErrCode === '-2') {
          this.setState({debugId: r.debugId}, () => {
            resolve()
          })
        } else {
          if (/列名\s*'[a-zA-Z0-9_-]+'\s*无效/.test(result.message)) {
            let tail = ''
            let type = ''
            if (setting.execute !== 'false' && setting.queryType !== 'statistics') {
              searches.forEach(item => {
                if (item.forbid) return
                item.key.split(',').forEach(field => {
                  if (new RegExp(`'${field}'`).test(result.message)) {
                    tail = field
                    type = '搜索条件'
                  }
                })
              })
            }

            if (!tail && setting.order) {
              let keys = setting.order.replace(/\s+(asc|desc)/ig, '').replace(/\s+/g, '')
              keys.split(',').forEach(field => {
                if (new RegExp(`'${field}'`).test(result.message)) {
                  tail = field
                  type = '排序'
                }
              })
            }

            if (!tail) {
              columns.forEach(item => {
                if (new RegExp(`'${item.field}'`).test(result.message)) {
                  tail = item.field
                  if (config.subtype === 'basetable') {
                    type = '显示列'
                  } else {
                    type = '字段集'
                  }
                }
              })
              if (!tail && config.subtype === 'dualdatacard') {
                subColumns.forEach(item => {
                  if (new RegExp(`'${item.field}'`).test(result.message)) {
                    tail = item.field
                    type = '子表字段集'
                  }
                })
              }
            }

            if (tail) {
              result.message = result.message.replace(/ROLLBACK TRANSACTION 请求没有对应的 BEGIN TRANSACTION。/, '')
              result.message = <>{result.message}<span style={{color: 'red'}}>注：{type}中存在字段{tail}</span></>
            }
          }
          if (type === 'submit') {
            Modal.confirm({
              title: result.message,
              okText: '知道了',
              cancelText: '强制保存',
              onOk: () => {
                reject()
              },
              onCancel() {
                resolve()
              }
            })
          } else {
            reject()
            Modal.error({
              title: result.message,
              okText: '知道了'
            })
          }
        }
      })
    } else {
      resolve()
    }
  }

  triggerConfirm = () => {
    const { script, scriptValue, scripts } = this.state
    let _scripts = fromJS(scripts).toJS()

    if (!scriptValue) {
      notification.warning({
        top: 92,
        message: '请输入sql!',
        duration: 5
      })
      return
    }

    let editLineId = ''
    if (script) {
      editLineId = script.uuid
      _scripts = _scripts.map(item => {
        if (script.uuid === item.uuid) {
          item.sql = scriptValue
        }
        return item
      })
    } else {
      let _script = {
        uuid: Utils.getuuid(),
        sql: scriptValue,
        $index: _scripts.length + 1,
        status: 'true'
      }

      editLineId = _script.uuid
      _scripts.push(_script)
    }

    this.setState({loading: true, editLineId})

    this.sqlverify(() => {this.setState({scripts: _scripts, script: null, scriptValue: '', loading: false})}, () => {this.setState({loading: false})}, 'script', _scripts)
  }

  updatefields = (columns) => {
    this.setState({
      columns: columns
    })
  }

  updateSubfields = (columns) => {
    this.setState({
      subColumns: columns
    })
  }

  copyDatasource = () => {
    const { config } = this.props
    const { columns, subColumns, scripts } = this.state

    this.settingForm.handleConfirm().then(res => {
      delete res.supModule
      res.name = config.name

      let source = {
        key: 'interface',
        type: 'line',
        data: {setting: res, columns, subColumns, scripts, pageable: false, format: 'array', name: res.name, status: 'false', type: 'interface', uuid: '' }
      }

      try {
        source = window.btoa(window.encodeURIComponent(JSON.stringify(source)))
      } catch (e) {
        console.warn('Stringify Failure')
        source = ''
      }
  
      if (source) {
        let oInput = document.createElement('input')
        oInput.value = source
        document.body.appendChild(oInput)
        oInput.select()
        document.execCommand('Copy')
        document.body.removeChild(oInput)
        message.success('复制成功。')
      }
    })
  }

  pasteSubmit = () => {
    const { config } = this.props
    const { setting } = this.state

    this.pasteFormRef.handleConfirm().then(res => {
      if (res.key !== 'interface') {
        notification.warning({
          top: 92,
          message: '配置类型错误!',
          duration: 5
        })
        return
      } else if (res.type !== 'line') {
        notification.warning({
          top: 92,
          message: '请粘贴单项数据源!',
          duration: 5
        })
        return
      }

      if (setting.supModule) {
        res.data.setting.supModule = setting.supModule
      }

      if (config.subtype !== 'basetable') {
        let subColumns = []

        if (config.subtype === 'dualdatacard' && res.data.subColumns) {
          subColumns = res.data.subColumns.map(col => {
            col.uuid = Utils.getuuid()
            return col
          })
        }

        this.setState({
          scripts: res.data.scripts.map(col => {
            col.uuid = Utils.getuuid()
            return col
          }),
          columns: res.data.columns.map(col => {
            col.uuid = Utils.getuuid()
            return col
          }),
          subColumns: subColumns,
          setting: res.data.setting,
          median: res.data.setting,
          reload: true,
          pvisible: false
        }, () => {
          this.setState({reload: false})
        })
      } else {
        this.setState({
          scripts: res.data.scripts.map(col => {
            col.uuid = Utils.getuuid()
            return col
          }),
          setting: res.data.setting,
          median: res.data.setting,
          reload: true,
          pvisible: false
        }, () => {
          this.setState({reload: false})
        })
      }

      this.props.updRecord({...res.data.setting})
    })
  }

  updateStatus = (res) => {
    this.setState({median: {...res}})
    this.props.updRecord({...res})
  }

  copyColumns = () => {
    const { config } = this.props
    const { columns, subColumns, setting } = this.state

    let _columns = columns
    let m = []
    let n = []
    let s = []

    if (subColumns && subColumns.length) {
      _columns = [...columns, ...subColumns]
    }

    _columns.forEach(col => {
      m.unshift(`${col.field} ${col.datatype}`)
      n.unshift(col.field)
      if (/decimal|int/ig.test(col.datatype)) {
        s.unshift(`${Math.round(Math.random() * 10)} as ${col.field}`)
      } else if (/datetime/.test(col.datatype)) {
        s.unshift(`'${moment().format('YYYY-MM-DD HH:mm:ss')}' as ${col.field}`)
      } else if (/date/.test(col.datatype)) {
        s.unshift(`'${moment().format('YYYY-MM-DD')}' as ${col.field}`)
      } else {
        s.unshift(`'${col.label}' as ${col.field}`)
      }
    })

    if (window.debugger) {
      window.mkInfo('select ' + s.join(', '))
    }

    let extend = ''
    if (config.hasExtend) {
      extend = []
      let mapCol = (cols) => {
        cols.forEach(col => {
          if (col.type === 'extend') {
            let datatype = 'Nvarchar(50)'
            _columns.forEach(c => {
              if (c.field === col.field) {
                datatype = c.datatype
              }
            })
            for (let i = 1; i < col.quota; i++) {
              extend.push(`${col.field}${i} ${datatype}`)
            }
          } else if (col.type === 'colspan' && col.subcols) {
            mapCol(col.subcols)
          }
        })
      }
      mapCol(config.cols)
      extend = `
      /* 扩展列字段 */
      /* ${extend.join(',')} */
      `
    }

    let oInput = document.createElement('input')
    oInput.value = `create table #${setting.tableName || 'tb'}
    (${m.join(',')},sort_id INT IDENTITY(1,1))
    insert into #${setting.tableName || 'tb'}
    (${n.join(',')})
    select ${n.join(',')}
    from ${setting.dataresource ? `(${setting.dataresource.replace(/\n/g, ' ')}) tb` : setting.tableName || 'tb'}
    order by @orderBy@
    
    declare @mk_total int
    set @mk_total = 0
      
    select @mk_total = count(1) from #${setting.tableName || 'tb'}
      declare @pageIndex_top int 
      set @pageIndex_top=(@pageIndex@-1)*@pageSize@ 
      
      if @mk_total > @pageIndex@*@pageSize@
        delete #${setting.tableName || 'tb'} where sort_id > @pageIndex@*@pageSize@ 
      
      if @pageIndex_top > 0
        delete #${setting.tableName || 'tb'} where sort_id <= @pageIndex_top

    drop table #${setting.tableName || 'tb'}${extend}`

    document.body.appendChild(oInput)
    oInput.select()
    document.execCommand('Copy')
    document.body.removeChild(oInput)

    message.success('复制成功。')
  }

  copySubColumns = () => {
    const { subColumns } = this.state
    let m = []
    let n = []

    subColumns.forEach(col => {
      m.push(`${col.field} ${col.datatype}`)
      n.push(col.field)
    })

    let oInput = document.createElement('input')
    oInput.value = `/*${m.join(',')}*/
      ${n.join(',')}`
    document.body.appendChild(oInput)
    oInput.select()
    document.execCommand('Copy')
    document.body.removeChild(oInput)

    message.success('复制成功。')
  }

  addProcess = () => {
    const { config } = this.props
    const { columns } = this.state

    let fields = []
    let cols = []
    let _cols = columns.map(item => item.field.toLowerCase())

    if (!_cols.includes('remark_w')) {
      fields.push({ uuid: Utils.getuuid(), label: '备注', field: 'remark_w', datatype: 'Nvarchar(50)', fieldlength: 50, type: 'text' })
      cols.push({ uuid: Utils.getuuid(), Width: 120, label: '备注', field: 'remark_w', type: 'text', Hide: 'false', IsSort: 'false', fieldlength: 50 })
    }
    if (!_cols.includes('statusname_w')) {
      fields.push({ uuid: Utils.getuuid(), label: '状态', field: 'statusname_w', datatype: 'Nvarchar(50)', fieldlength: 50, type: 'text' })
      cols.push({ uuid: Utils.getuuid(), Width: 120, label: '状态', field: 'statusname_w', type: 'text', Hide: 'false', IsSort: 'false', fieldlength: 50 })
    }
    if (!_cols.includes('works_flow_param')) {
      fields.push({ uuid: Utils.getuuid(), label: '流程参数', field: 'works_flow_param', datatype: 'Nvarchar(512)', fieldlength: 512, type: 'text' })
      cols.push({ uuid: Utils.getuuid(), Width: 120, label: '流程参数', field: 'works_flow_param', type: 'text', Hide: 'true', IsSort: 'false', fieldlength: 512 })
    }

    if (fields.length === 0) return

    const that = this

    if (config.subtype !== 'basetable') {
      cols = null
    }

    confirm({
      content: cols ? '显示列中是否添加工作流字段？' : '字段集中是否添加工作流字段？',
      onOk() {
        that.setState({
          cols: cols,
          columns: [...fields, ...columns]
        })
      },
      onCancel() {}
    })
  }

  updateColumns = (tableName) => {
    const { config } = this.props
    const { colLoading, columns } = this.state

    if (colLoading || columns.length > 0) return

    if (config.subtype === 'basetable') return

    let param = {func: 'sPC_Get_FieldName', TBName: tableName}
    if (window.GLOB.cloudServiceApi) { // 且存在云端地址
      param.rduri = window.GLOB.cloudServiceApi
      param.userid = sessionStorage.getItem('CloudUserID') || ''
      param.LoginUID = sessionStorage.getItem('CloudLoginUID') || ''
    }

    this.setState({colLoading: true})

    Api.getSystemCacheConfig(param).then(result => {
      this.setState({colLoading: false})
      if (!result.status) return

      let fields = []
      result.FDName.forEach(item => {
        if (item.f_type !== 0) return

        if (/nvarchar|int|decimal/ig.test(item.FieldType) || item.FieldType === 'date' || item.FieldType === 'datetime') {
          fields.unshift({
            uuid: Utils.getuuid(),
            label: item.FieldDec,
            field: item.FieldName,
            datatype: item.FieldType.toLowerCase()
          })
        }
      })

      if (fields.length === 0) return

      this.setState({columns: fields})
    }, () => {
      this.setState({colLoading: false})
    })
  }

  tolowercase = (type) => {
    const that = this
    confirm({
      content: type === 'sub' ? '确定将子表字段转为小写吗？' : '确定将字段转为小写吗？',
      onOk() {
        that.execlowercase(type)
      },
      onCancel() {}
    })
  }

  execlowercase = (type) => {
    const { subColumns, columns } = this.state

    if (type === 'sub') {
      this.setState({subColumns: fromJS(subColumns).toJS().map(col => {
        col.field = col.field.toLowerCase()
        return col
      })})
    } else {
      this.setState({columns: fromJS(columns).toJS().map(col => {
        col.field = col.field.toLowerCase()
        return col
      })})
    }
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  render() {
    const { config } = this.props
    const { columns, subColumns, median, setting, scripts, colColumns, scriptsColumns, activeKey, loading, searches, defaultsql, visible, pvisible, reload, script, scriptValue, searchKey, editLineId } = this.state

    return (
      <div className="model-data-source-wrap">
        {config.name ? <div className="mk-com-name">{config.name} - 数据源</div> : null}
        {loading && <Spin size="large" />}
        <Tabs activeKey={activeKey} className="data-source-card-box" onChange={this.changeTab}>
          <TabPane tab={
            <span>
              数据源
            </span>
          } key="setting">
            {!reload ? <SettingForm
              config={config}
              columns={columns}
              subColumns={subColumns}
              setting={setting}
              scripts={scripts}
              updateStatus={this.updateStatus}
              updateColumns={this.updateColumns}
              addProcess={this.addProcess}
              wrappedComponentRef={(inst) => this.settingForm = inst}
            /> : null}
          </TabPane>
          {!['basetable', 'invoice', 'invTable'].includes(config.subtype) ? <TabPane tab={
            <span>
              字段集
              {columns.length ? <span className="count-tip">{columns.length}</span> : null}
            </span>
          } key="columns">
            <ColForm columnChange={this.columnChange}/>
            <FieldsComponent
              config={{columns}}
              type="fields"
              updatefield={this.updatefields}
            />
            <SwapOutlined className="columns-lowercase" title="转小写" onClick={() => this.tolowercase()}/>
            <ExcelOut data={columns} setting={setting}/>
            <EditTable actions={['edit', 'move', 'copy', 'del', 'clear']} searchKey={searchKey} type="datasourcefield" wrappedComponentRef={(inst) => this.datasource = inst} data={columns} columns={colColumns} onChange={(columns) => this.setState({columns})}/>
          </TabPane> : <TabPane tab={
            <span>
              字段集<TableOutlined style={{marginLeft: '5px', fontSize: '12px'}} />
              {columns.length ? <span className="count-tip">{columns.length}</span> : null}
            </span>
          } key="columns">
            <div className="base-table-columns"></div>
            <ExcelOut data={columns} setting={setting}/>
            <EditTable actions={[]} searchKey={searchKey} type="datasourcefield" data={columns} columns={colColumns}/>
          </TabPane>}
          {config.subtype === 'dualdatacard' ? <TabPane tab={
            <span>
              子表字段集
              {subColumns.length ? <span className="count-tip">{subColumns.length}</span> : null}
            </span>
          } key="subcolumns">
            <ColForm columnChange={this.subColumnChange}/>
            <FieldsComponent
              config={{columns: subColumns}}
              type="fields"
              updatefield={this.updateSubfields}
            />
            <SwapOutlined className="columns-lowercase" title="转小写" onClick={() => this.tolowercase('sub')}/>
            <ExcelOut data={subColumns} setting={setting}/>
            <EditTable actions={['edit', 'move', 'copy', 'del', 'clear']} searchKey={searchKey} type="datasourcefield" wrappedComponentRef={(inst) => this.subdatasource = inst} data={subColumns} columns={colColumns} onChange={(subColumns) => this.setState({subColumns})}/>
          </TabPane> : null}
          <TabPane tab={
            <span>
              自定义脚本
              {scripts.length ? <span className="count-tip">{scripts.length}</span> : null}
              {activeKey === 'setting' ? <CopyOutlined title="复制数据源" className="mk-copy-datasource" onClick={(e) => {e.stopPropagation();this.copyDatasource()}}/> : null}
              {activeKey === 'setting' ? <SnippetsOutlined title="导入数据源" className="mk-paste-datasource" onClick={(e) => {e.stopPropagation();this.setState({pvisible: true})}}/> : null}
              {activeKey === 'columns' ? <CopyOutlined title="以逗号拼接形式复制字段" className="mk-copy-fields" onClick={(e) => {e.stopPropagation();this.copyColumns()}}/> : null}
              {activeKey === 'subcolumns' ? <CopyOutlined title="以逗号拼接形式复制字段" className="mk-copy-fields" onClick={(e) => {e.stopPropagation();this.copySubColumns()}}/> : null}
              {activeKey === 'subcolumns' || activeKey === 'columns' ? <span onClick={(e) => {e.stopPropagation()}}><Search className="mk-search-fields" defaultValue={searchKey} allowClear onSearch={(val, e) => {e.stopPropagation();this.setState({searchKey: val})}} /></span> : null}
            </span>
          } key="scripts" disabled={median.interType !== 'system'} id="mk-scripts-tabpane">
            {scripts.length ? <BorderOutlined className="full-scripts" onClick={() => {
              if (this.scriptsForm && (this.scriptsForm.state.editItem || (this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))))) {
                notification.warning({
                  top: 92,
                  message: '请保存自定义脚本！',
                  duration: 5
                })
                return
              }
              this.setState({visible: true, script: null, scriptValue: '', editLineId: ''})
            }}/> : null}
            <CustomScriptsForm
              type={config.type}
              hasExtend={config.hasExtend}
              setting={setting}
              searches={searches}
              defaultsql={defaultsql}
              customScripts={scripts}
              systemScripts={this.state.systemScripts}
              scriptsChange={this.scriptsChange}
              scriptSubmit={this.scriptSubmit}
              addProcess={this.addProcess}
              wrappedComponentRef={(inst) => this.scriptsForm = inst}
            />
            <EditTable actions={['move']} data={scripts} columns={scriptsColumns} onChange={this.changeScripts}/>
          </TabPane>
        </Tabs>
        <Modal
          wrapClassName="model-custom-view-scripts-modal"
          title="自定义脚本"
          visible={visible}
          width={'95vw'}
          maskClosable={false}
          destroyOnClose
        >
          <img className="unfull-scripts" src={MinView} onClick={() => this.setState({visible: false, script: null})} alt=""/>
          <div className="script-table-wrap">
            {scripts.map(item => {
              let title = item.sql.match(/^\s*\/\*.+\*\//)
              title = title && title[0] ? title[0] : ''
              let _text = title ? item.sql.replace(title, '') : item.sql

              if (item.status === 'false') {
                return (
                  <div className="script-item" key={item.uuid}>
                    <div style={{cursor: 'not-allowed'}}>
                      {title ? <div style={{color: '#a50', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>{title}</div> : null}
                      <Paragraph copyable={{ text: item.sql }} ellipsis={{ rows: 4 }}>{_text}</Paragraph>
                      <div>
                        <span style={{color: '#ff4d4f', marginLeft: '20px'}}>
                          禁用
                          <StopOutlined style={{marginLeft: '5px'}} />
                        </span>
                      </div>
                    </div>
                    <div style={{height: '24px'}}></div>
                  </div>
                )
              } else {
                let sign = ''
                if (script && script.uuid === item.uuid) {
                  sign = 'active'
                } else if (editLineId === item.uuid) {
                  sign = 'edited'
                }
                return (
                  <div className={'script-item ' + sign} key={item.uuid}>
                    <div style={{cursor: 'pointer'}} onClick={() => {
                      this.setState({script: item, scriptValue: item.sql})
                    }}>
                      {title ? <div style={{color: '#a50', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>{title}</div> : null}
                      <Paragraph copyable={{ text: item.sql }} ellipsis={{ rows: 4 }}>{_text}</Paragraph>
                      <div>
                        <span style={{color: '#26C281', marginLeft: '20px'}}>
                          启用
                          <CheckCircleOutlined style={{marginLeft: '5px'}}/>
                        </span>
                      </div>
                    </div>
                    <div style={{textAlign: 'right'}}>
                      <span className="operation-btn" onClick={() => this.handleStatus(item)} style={{color: '#8E44AD'}}><SwapOutlined /></span>
                      <Popconfirm
                        overlayClassName="popover-confirm"
                        title="确定删除吗?"
                        onConfirm={() => this.deleteScript(item)
                      }>
                        <span className="operation-btn" style={{color: '#ff4d4f'}}><DeleteOutlined /></span>
                      </Popconfirm>
                    </div>
                  </div>
                )
              }
            })}
          </div>
          <div className="script-button">
            <Button onClick={this.triggerConfirm} loading={this.state.loading} className="mk-green" style={{marginBottom: 15, marginLeft: 40}}>
              {script ? '保存' : '添加'}
            </Button>
            <Button onClick={() => {this.setState({script: null, scriptValue: ''})}} style={{marginBottom: 15, marginLeft: 10}}>
              取消
            </Button>
          </div>
          <CodeMirror value={scriptValue} onChange={(val) => {this.setState({scriptValue: val})}}></CodeMirror>
        </Modal>
        <Modal
          title="粘贴"
          visible={pvisible}
          width={600}
          maskClosable={false}
          onOk={this.pasteSubmit}
          onCancel={() => {this.setState({pvisible: false})}}
          destroyOnClose
        >
          <PasteForm wrappedComponentRef={(inst) => this.pasteFormRef = inst} inputSubmit={this.pasteSubmit}/>
        </Modal>
      </div>
    )
  }
}

export default Form.create()(VerifyCard)